#
# This script implements a simple remote-control mechanism for
# Tk applications.  It allows you to select an application and
# then type commands to that application.
#

set inputcolour  green4
set outputcolour red
set insertcolour blue
set bkgcolour    azure

set inputprompt  ""
set outputprompt ""
set insertprompt ">>"

set histlim 40
for {set i 0} {$i <= $histlim} {incr i} {lappend history ""}
set nextcommand $histlim

# The global variable below keeps track of the remote application
# that we're sending to.  If it's an empty string then we execute
# the commands locally.

set app "local"

# The global variable below keeps track of whether we're in the
# middle of executing a command entered via the text.

set executing 0 

# The global variable below keeps track of the last command executed,
# so it can be re-executed in response to !! commands.

set debug 0
set started 0

set frame RMTCNTRL
set frametitle "Tk Remote Controller"
set icontitle "Tk Remote"

set window_height 0

proc start args {
   global started 
   if {$started == 0} then {make-frame} else {open-frame}
   set started 1
   redisplay
}

proc click-quit args {close-frame}

proc finish args {destroy-frame}

proc make-frame {} {
   global env app window_height
   global nextcommand

set rows 20
set cols 80

  :set footer_position center
  :set autoreturn "F"

  :frame [self frame-name] :label "[self frame-title]" :show-footer T  \
    :icon-name "[self icon-title]"

   # Create menu bar.  Arrange to recreate all the information in the
   # applications sub-menu whenever it is cascaded to.

    :panel M :layout T 
    :layout H
    :menubutton MENU      :label "Applications"
    bind [:path MENU] <Enter> "redisplay"
    :text APPL            :label "Current Application" :w 16
    :menubutton COMMANDS  :label "Commands"
    :menubutton HISTORY   :label "History"  :strings [list ""]
    :button CLEAR 
#    :button HELP          :label "Help"

#    create command menu

    set menu {}
    set file $env(TCLLOCAL)/midas-rmtcommands
    if {[file exists $file]} then {
       set f [open $file r]
       set l [read $f]
       close $f
       set sys {system}
       foreach item $l {lappend sys $item}
       lappend menu $sys
    }
    set file $env(HOME)/.midas-rmtcommands
    if {[file exists $file]} then {
       set f [open $file r]
       set l [read $f]
       close $f
       set user {user}
       foreach item $l {lappend user $item}
       lappend menu $user
    }
    :modify COMMANDS :menu $menu

   # Create input window

    :panel C :layout B 
    :colour local entry foreground black
    :text CMD :label "Input:" :x 12 :bold T :size L :w [expr $cols - 12]
    bind [:path CMD].entry <Meta-k>         "restorecommand last"
    bind [:path CMD].entry <Meta-BackSpace> "zapcommand"
    bind [:path CMD].entry <Alt-KeyPress-k>         "restorecommand last"
    bind [:path CMD].entry <Alt-KeyPress-BackSpace> "zapcommand"
    bind [:path CMD].entry <Up> "restorecommand previous"
    bind [:path CMD].entry <Down> "restorecommand next"

   # Create text window and scrollbar.

    :panel L :layout T 
    :galley LIST :rows $rows :cols $cols :font variable
    configuretags

    open-frame

   :galley_size LIST $rows $cols

    focus [:path CMD].entry

    set app [Get_My_Name]
}

proc click-configure {W} {
   :galley_resize LIST
}

proc restorecommand {c} {
  global history histlim nextcommand 
  switch $c {
    last  {:modify CMD :value [lindex $history end]}
    previous  {
     :modify CMD :value [lindex $history $nextcommand]
     incr nextcommand -1
     if {$nextcommand < 0} then {set nextcommand 0}
    }
    next  {
     incr nextcommand 1
     if {$nextcommand > $histlim} then {set nextcommand $histlim}
     :modify CMD :value [lindex $history $nextcommand]
    }
    default {insert "" "restorecommand $c?"}
  }
}

proc zapcommand {} {:modify CMD :value ""}

proc configuretags {} {
   global inputcolour outputcolour insertcolour bkgcolour

    :modify LIST :createtag "taginput  -foreground $inputcolour  -background $bkgcolour"
    :modify LIST :createtag "tagoutput -foreground $outputcolour -background $bkgcolour"
    :modify LIST :createtag "taginsert -foreground $insertcolour -background $bkgcolour"
}

proc redisplay {} {
  global app

  :modify MENU :strings [linsert [Get_Task_Names] 0 local]
  :modify APPL :value $app
}

proc click arg {
  global debug app history
   if {$debug > 0} then {insert "" "proc click called with args $arg"}

   set f [lindex $arg 0]
   set c [lindex $arg 1]
   set w [lindex $arg 2]
   set v [lindex $arg 3]
   if {$c==":quit"} then {click-quit; return}

   switch $w  {
      MENU       {set app "$v"; :modify APPL :value $app}
      HELP       {}
      CLEAR      {:modify LIST :empty T}

      CMD        -
      COMMANDS   {invoke "$v"}

      HISTORY    {
                 set i [first $v]
                 invoke "[lindex $history $i]"
                 }

      default    {puts stdout $arg}
   }
}


# The procedure below executes a command (it takes everything on the
# current line after the prompt and either sends it to the remote
# application or executes it locally, depending on "app".

proc invoke {cmd} {
    global app executing
    global outputprompt inputprompt
    global history histmenu histlim nextcommand

    :modify LIST :usingtag taginput :strings "[set inputprompt]$cmd\n"

    incr executing 1
    if [info complete $cmd] {
        set-footer "Sending to $app"
        set-busy
	if {$cmd == "!!"} {
	    set cmd [lindex $history end]
	} else {
            lappend history "$cmd"
            set history [lreplace $history 0 0]
	}
	if {$app == "local"} {
	    set result [catch [list uplevel #0 $cmd] msg]
	} else {
	    set result [catch {inform $app $cmd} msg]
	}
	if {$result != 0} {
	     :modify LIST :usingtag tagoutput :strings "[set outputprompt]Error: $msg\n"
	} else {
	    if {$msg != ""} {
                :modify LIST :usingtag tagoutput :strings "$outputprompt$msg\n"
	    }
	}
        clear-busy
        clear-footer
    }
    incr executing -1

    :modify CMD :value ""
    set nextcommand $histlim

    set histmenu {}
    set i 0
    foreach s $history {
       if {$s != ""} then {lappend histmenu "$i [string range "$s" 0 40]"}
       incr i
    }
    :modify HISTORY :strings $histmenu
}

proc insert {i s} {
  global insertprompt
  :modify LIST :usingtag taginsert :strings "$i$insertprompt$s\n"
}
